from dataclasses import field
from typing import List
import github.com.metaprov.modelaapi.pkg.apis.catalog.v1alpha1.generated_pb2 as catalog_pb
import github.com.metaprov.modelaapi.pkg.apis.inference.v1alpha1.generated_pb2 as inference_pb
from github.com.metaprov.modelaapi.services.grpcinferenceservice.v1.grpcinferenceservice_pb2 import *
from modela.Configuration import datamodel
from modela.common import Configuration, ConditionStatus, Time, ObjectReference, StatusError, TriggerScheduleEventType
from modela.inference.common import PredictorConditionType, CanaryMetric, AccessType, PredictorType, ModelDeploymentPhase
from modela.infra.models import Workload
from modela.data.models import DataLocation
from modela.training.common import TaskType
from modela.training.models import ModelValidation, ModelValidationResult


@datamodel(proto=catalog_pb.RunSchedule)
class RunSchedule(Configuration):
    """ RunSchedule specifies the schedule for a Job to be executed """
    Enabled: bool = False
    """ Indicates if the schedule is enabled and the Jobs associated it will be created at the specified time """
    StartTime: Time = None
    """ The time of the day when the schedule will be executed """
    EndTime: Time = None
    """ The time of the day when the schedule is expected to conclude """
    Cron: str = ''
    """ The cron string of the schedule. See https://docs.oracle.com/cd/E12058_01/doc/doc.1014/e12030/cron_expressions.htm for more information """
    Type: TriggerScheduleEventType = None
    """ The type of schedule, which can be a frequency interval or a cron expression """


@datamodel(proto=inference_pb.ModelDeploymentStatus)
class ModelDeploymentStatus(Configuration):
    ImageName: str = ''
    DeploymentRef: ObjectReference = None
    """ The deployment name that serves this model """
    ServiceRef: ObjectReference = None
    """ The service name that serves this model """
    HpaName: str = ''
    """ the name of the horizonal pod autoscaler, if autoscaling is true """
    P50: float = 0
    """ P50 latency """
    P95: float = 0
    """ P95 latency """
    P99: float = 0
    """ P99 is the 99% latency of the model """
    LastPredictionTime: Time = None
    """ Last prediction time is the time of the last prediction """
    DailyPredictionAvg: int = 0
    LastFailure: str = ''
    """ LastFailure is the last failure that occur with the model """
    Phase: ModelDeploymentPhase = None
    """ Phase is the current phase of this model deployment """
    DeployedAt: Time = None
    """ DeployedAt is the last time that this model was deployed """
    ReleasedAt: Time = None
    """ ReleasedAt is the time that this model was released """
    DeploymentReady: bool = False
    """ If true, the deployment is ready """
    ServiceReady: bool = False
    """ If true, the service is ready """
    DataDrift: bool = False
    """ Indicates if a data drift has been detected based on incoming prediction data """
    ConceptDrift: bool = False
    """ Indicates if a concept drift has been detected based on incoming prediction data """
    LastDailyPredictions: List[int] = field(default_factory=lambda : [])
    """ The predictions from the last 7 days """


@datamodel(proto=inference_pb.ModelRecord)
class ModelRecord(Configuration):
    """ ModelRecord hold the state of a model that was in production """
    ModelName: str = ''
    """ Model Name is the name of the model """
    ModelVersion: str = ''
    """ Model version is the version of the model """
    LiveAt: Time = None
    """ Live at is the time that the model were placed in production """
    RetiredAt: Time = None
    """ Retried at is the time that the model was retired from production. """
    AvgDailyPrediction: int = 0
    """ Retried at is the time that the model was retired from production. """
    AvgLatency: float = 0
    """ Retried at is the time that the model was retired from production. """


@datamodel(proto=catalog_pb.ModelDeploymentSpec)
class ModelDeploymentSpec(Configuration):
    """
    ModelDeploymentSpec describes how a single model should be deployed with a Predictor, and
    how prediction traffic will be routed to the model
    """
    ModelName: str = ''
    """
    The name of a model, which is fully complete and packaged, that exists in the same DataProduct namespace
    as the resource which specifies the ModelDeploymentSpec
    """
    ModelVersion: str = ''
    """ The version of the model """
    MaxTraffic: int = 0
    """ The maximum percentage of traffic that will be served by the model """
    Traffic: int = 0
    """ The minimum percentage of traffic that will be served by the model """
    Canary: bool = False
    """ Canary denotes if this deployment is a staged release. A staged release will serve traffic in increments """
    Shadow: bool = False
    """
    Shadow denotes if the model is running in shadow mode. A shadow model face the production traffic, however, the predictions are not
    served back to the client
    """
    Released: bool = False
    """ A released model is a model that should serve production traffic """
    Deployed: bool = False
    """ A deployed model is a model whose containers are up, but does not serve production traffic """
    MountTar: bool = False
    """ MountTar means that we would mount the model tar file. Else we would use baked image. """
    TrafficSelector: str = ''
    """ TrafficSelector is a filter on the traffic to this model """
    CanaryMetrics: List[CanaryMetric] = field(default_factory=lambda : [])
    """
    If the deployment is canary, the metric define how to evaluate the canary.
    Default: none
    """
    ApprovedBy: str = ''
    """ The account name of the approver """
    ApprovedAt: Time = None
    """ The time of approval """


@datamodel(proto=inference_pb.PredictorCondition)
class PredictorCondition(Configuration):
    """ PredictorCondition describes the state of a Predictor at a certain point """
    Type: PredictorConditionType = None
    """ Type of Predictor condition """
    Status: ConditionStatus = None
    """ Status of the condition, one of True, False, Unknown """
    LastTransitionTime: Time = None
    """ Last time the condition transitioned from one status to another """
    Reason: str = ''
    """ The reason for the condition's last transition """
    Message: str = ''
    """ A human-readable message indicating details about the transition """


@datamodel(proto=inference_pb.ProgressiveSpec)
class ProgressiveSpec(Configuration):
    """ ProgressiveSpec defines the specification to progressively deploy a model to production """
    Warmup: int = 0
    """ The time, in seconds, for the warm-up period """
    TrafficIncrement: int = 0
    """ The percentage of traffic to increment """
    CanaryMetrics: List[CanaryMetric] = field(default_factory=lambda : [])
    """ What metric to use when comparing the candidate model to the current model """


@datamodel(proto=inference_pb.PredictionCacheSpec)
class PredictionCacheSpec(Configuration):
    """ PredictionCacheSpec specifies the connection information of a key-value cache to store predictions """
    Active: bool = False
    """ Active indicate if the cache is active """
    ServiceName: str = ''
    """ the name of the cache service """


@datamodel(proto=inference_pb.AutoScaling)
class AutoScaling(Configuration):
    """ AutoScaling defines the configuration for the automatic scaling of a service """
    Enabled: bool = False
    """ Indicates if automatic scaling is enabled """
    MinReplicas: int = 1
    """ The minimum number of replicas running the service """
    MaxReplicas: int = 1
    """ The maximum number of replicas running the service """
    CpuAvgUtilization: int = 80
    """
    The target average CPU utilization across all replicas. The HorizontalPodAutoscaler associated with the
    service will change the number of replicas to maintain this metric
    """
    MemAvgUtilization: int = 80
    """
    The target average memory utilization across all replicas. The HorizontalPodAutoscaler associated with the
    service will change the number of replicas to maintain this metric
    """


@datamodel(proto=inference_pb.PredictorStatus)
class PredictorStatus(Configuration):
    """ PredictorStatus contain the current state of the Predictor resource """
    ObservedGeneration: int = 0
    """ ObservedGeneration is the last generation that was acted on """
    History: List[ModelRecord] = field(default_factory=lambda : [])
    """ The collection of historical records of models deployed to the Predictor, used internally to roll-back models """
    LastUpdated: Time = None
    """ The last time the object was updated """
    EndPoint: str = ''
    """ The end-point URL of the Predictor """
    FailureReason: StatusError = None
    """ In the case of failure, the Predictor resource controller will set this field with a failure reason """
    FailureMessage: str = ''
    """ In the case of failure, the Predictor resource controller will set this field with a failure reason """
    Conditions: List[PredictorCondition] = field(default_factory=lambda : [])


@datamodel(proto=inference_pb.MonitorSpec)
class MonitorSpec(Configuration):
    """ MonitorSpec defines the specification to monitor a model in production """
    Enabled: bool = False
    """ If true monitoring is enabled. """
    SamplePercent: int = 0
    """ How many rows to sample from the live data for """
    Schedule: RunSchedule = None
    """ Schedule for running the monitor """
    NotifierName: str = ''
    """ NotifierName is the name of notifier to alert in case of """
    Validations: List[ModelValidation] = field(default_factory=lambda : [])
    """ List of model validation """
    LogRequests: bool = False
    """ Log requests (incoming traffic) """
    LogResponses: bool = False
    """ Log outgoing traffic """


@datamodel(proto=inference_pb.MonitorStatus)
class MonitorStatus(Configuration):
    MonitorLastAttemptAt: Time = None
    MonitorLastScore: float = 0
    """ The score from the last time model monitoring was computed """
    MonitorLastLatency: float = 0
    """ The model latency from the last time model monitoring was computed """


@datamodel(proto=inference_pb.PredictorAuthSpec)
class PredictorAuthSpec(Configuration):
    Enabled: bool = False


AccessType_ = AccessType
AutoScaling_ = AutoScaling


@datamodel(proto=inference_pb.PredictorSpec)
class PredictorSpec(Configuration):
    """ PredictorSpec defines the desired state of a Predictor """
    VersionName: str = ''
    """
    The name of the DataProductVersion which describes the version of the resource
    that exists in the same DataProduct namespace as the resource
    """
    Description: str = ''
    """ The user-provided description of the Predictor """
    ProductRef: ObjectReference = None
    """ The reference to the DataProduct that the resource exists under """
    ServingsiteRef: ObjectReference = ObjectReference('default-tenant', 'default-lab')
    """ The reference to the ServingSite resource that hosts the Predictor """
    Models: List[ModelDeploymentSpec] = field(default_factory=lambda : [])
    """
    The collection of model deployment specifications that define which Model resources will be deployed to the
    Predictor service and how they will be deployed. Each model should be trained with the same type of
    dataset and possess a unique version
    """
    Progressive: ProgressiveSpec = None
    """
    The specification to progressively deploy models. ModelDeploymentSpec specifications within Models that have the
    `Canary` field enabled will be progressively deployed according to the specification when they are applied to the Predictor
    """
    ArtifactsFolder: DataLocation = DataLocation()
    """ The data location where artifacts generated by the Predictor will be stored """
    Port: int = 8080
    """
    The port number that will be exposed on the Predictor's Pods to serve prediction traffic through the GRPCInferenceService API.
    The Kubernetes Service created by the Predictor will expose the port and forward GRPC traffic to the backend pods
    """
    NodePort: int = 30000
    """
    The port number that will be exposed on the external address of every node on the cluster, in the case of the
    Predictor's access type being NodePort. Traffic from the port will be forwarded to the Predictor's backend service
    """
    Path: str = ''
    """
    The auto-generated DNS path where the Predictor service can be accessed. If the access type is ClusterIP, it will
    be a cluster-internal DNS name (i.e. predictor.default-serving-site.svc.cluster.local). In the case of the Ingress
    access type, it will be determined by the FQDN of the host ServingSite (i.e. predictor.default-serving-site.modela.ai).
    """
    AccessType: AccessType_ = None
    """
    The Kubernetes-native access method which specifies how the Kubernetes Service created by the Predictor will be exposed.
    See https://modela.ai/docs/docs/serving/production/#access-method for a detailed description of each access type
    (defaults to cluster-ip)
    """
    Replicas: int = 1
    """
    The number of replicas for the Kubernetes Deployment associated with the Predictor, which will instantiate multiple
    copies of the service in the case that automatic scaling is disabled
    """
    AutoScaling: AutoScaling_ = None
    """ AutoScaling specifies the auto-scaling policy """
    Owner: str = 'no-one'
    """ The name of the Account which created the object, which exists in the same tenant as the object """
    Resources: Workload = Workload('general-nano')
    """ Resources specifies the resource requirements allocated to the prediction service """
    Cache: PredictionCacheSpec = None
    """ Cache is the specification of prediction cache (currently unimplemented) """
    ForwardCurtain: str = ''
    """ The forward curtain receives prediction requests before the prediction (currently unimplemented) """
    BackwardCurtain: str = ''
    """ The backward curtain receives prediction requests after the prediction (currently unimplemented) """
    TargetColumn: str = ''
    """ The target feature of the model that the Predictor serves """
    PositiveLabel: str = ''
    """ For binary classification, the name of the positive class of the target feature """
    NegativeLabel: str = ''
    """ For binary classification, the name of the negative class of the target feature """
    TrainingDatasetRef: ObjectReference = None
    """ The dataset where this model was trained on """
    REST: bool = True
    Type: PredictorType = PredictorType.Online
    """ The type of predictor (online, batch, or streaming). Online is the only supported type as of the current release """
    Task: TaskType = None
    """ The task type of the Predictor, which should match the task type of the models being served """
    PredictionThreshold: float = 0
    """ The prediction threshold """
    Monitor: MonitorSpec = None
    """ Monitor spec specify the monitor for this predictor. """
    Auth: PredictorAuthSpec = None
    """ The specification to authenticate requests to the prediction service """


@datamodel(proto=ProbabilityValue)
class ProbabilityValue(Configuration):
    Label: str = ''
    Probability: float = 0


@datamodel(proto=ShapValue)
class ShapValue(Configuration):
    Feature: str = ''
    Value: float = 0


@datamodel(proto=PredictResultLineItem)
class PredictionResult(Configuration):
    Success: bool = False
    Score: float = 0
    Label: str = ''
    Probabilities: List[ProbabilityValue] = field(default_factory=lambda : [])
    ShapValues: List[ShapValue] = field(default_factory=lambda : [])
    MissingColumns: List[str] = field(default_factory=lambda : [])
    OutOfBound: List[str] = field(default_factory=lambda : [])
    BaseShapValue: float = 0



import grpc
from github.com.metaprov.modelaapi.pkg.apis.data.v1alpha1.generated_pb2 import Recipe as MDRecipe
from github.com.metaprov.modelaapi.services.recipe.v1.recipe_pb2_grpc import RecipeServiceStub
from github.com.metaprov.modelaapi.services.recipe.v1.recipe_pb2 import CreateRecipeRequest, \
    UpdateRecipeRequest, \
    DeleteRecipeRequest, GetRecipeRequest, ListRecipesRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class Recipe(Resource):
    def __init__(self, item: MDRecipe = MDRecipe(), client=None, namespace="", name="", version=Resource.DefaultVersion):
        super().__init__(item, client, namespace=namespace, name=name, version=version)


class RecipeClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: RecipeServiceStub = stub

    def create(self, recipe: Recipe) -> bool:
        request = CreateRecipeRequest()
        request.recipe.CopyFrom(recipe.raw_message)
        try:
            response = self.__stub.CreateRecipe(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, recipe: Recipe) -> bool:
        request = UpdateRecipeRequest()
        request.recipe.CopyFrom(recipe.raw_message)
        try:
            self.__stub.UpdateRecipe(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[Recipe, bool]:
        request = GetRecipeRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetRecipe(request)
            return Recipe(response.recipe, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteRecipeRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteRecipe(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[Recipe], bool]:
        request = ListRecipesRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListRecipes(request)
            return [Recipe(item, self) for item in response.recipes.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





from enum import Enum


class ProviderName(Enum):
    AmazonAthena = "athena"
    AmazonRedshift = "redshift"
    ApacheDrill = "drill"
    ApacheDruid = "druid"
    ApacheHive = "hive"
    ApacheImpala = "impala"
    ApacheKylin = "kylin"
    ApachePinot = "pinot"
    ApacheSpark = "spark"
    AzureSqlDatabase = "azuresqldatabase"
    GcpBigQuery = "bigquery"
    ApacheCassandra = "cassandra"
    CockrouchDB = "cockrouchdb"
    Clickhouse = "clickhouse"
    Couchbase = "couchbase"
    Dremio = "dremio"
    DB2 = "db2"
    ElasticSearch = "elasticsearch"
    Exasol = "exasol"
    Kafka = "kafka"
    GoogleSheets = "googlesheets"
    GcpSpanner = "spanner"
    Greenplum = "greenplum"
    MySql = "mysql"
    MongoDb = "mongodb"
    Oracle = "oracle"
    PostgresSQL = "postgres"
    Presto = "presto"
    SAPHana = "hana"
    Snowflake = "snowflake"
    Singlestore = "singlestore"
    Sybase = "sybase"
    MSSqlServer = "mssqlserver"
    Sqlite = "sqlite"
    RabbitMQ = "rabbitmq"
    Teradata = "teradata"
    Vertica = "vertica"
    Odbc = "odbc"
    Informix = "informix"
    Neo4j = "neo4j"
    AliCloud = "alicloud"
    Azure = "azure"
    DigitalOcean = "digitalocean"
    Hetzner = "hetzner"
    Linode = "linode"
    Gcp = "gcp"
    Minio = "minio"
    Ovh = "ovh"
    Aws = "aws"
    Openstack = "openstack"
    FTP = "ftp"
    SFTP = "sftp"
    Dropbox = "dropbox"
    Box = "box"
    Dockerhub = "dockerhub"
    ImageRegistry = "image-registry"
    GitHub = "github"
    Bitbucket = "bitbucket"
    Gitlab = "gitlab"
    SmtpProvider = "smtp"
    Slack = "slack"
    Hipchat = "hipchat"
    Discord = "discord"
    PageDuty = "pagerduty"
    PushOver = "pushover"
    OpsGenie = "opsgenie"
    Webhook = "webook"
    VictorOps = "victorops"
    Flock = "flock"
    MSTeam = "msteam"
    MatterMost = "mattermost"
    Facebook = "facebook"
    Twitter = "twitter"
    UnknownProvider = "unknown"


class ConnectionCategory(Enum):
    General = "general"
    CloudProvider = "cloud"
    DockerImageRegistry = "docker-image-registry"
    Database = "database"
    Git = "git"
    Messaging = "messaging"
    ConnectionMessagerBroker = "message-broker"
    ConnectionGraphDatabase = "graph-database"
    ConnectionSocialMedia = "social-media"


class ConnectionConditionType(Enum):
    ConnectionReady = "Ready"
    ConnectionSaved = "Saved"


class Resource(Enum):
    CPU = "cpu"
    Memory = "memory"
    Storage = "storage"


class Verb(Enum):
    All = "*"
    Get = "get"
    List = "list"
    Watch = "watch"
    Create = "create"
    Update = "update"
    Patch = "patch"
    Delete = "delete"


class ResourceKind(Enum):
    Algorithm = "algorithms"
    Cloud = "clouds"
    MLFramework = "mlframeworks"
    PretrainedModel = "pretrainedmodels"
    PublicDataset = "publicdatasets"
    UserRoleClass = "userroleclasses"
    WorkloadClass = "workloadclasses"

    DataPipeline = "datapipelines"
    DataPipelineRun = "datapipelineruns"
    DataProduct = "dataproducts"
    DataProductVersion = "dataproductversions"
    Dataset = "datasets"
    DataSource = "datasources"
    Entity = "entities"
    Feature = "features"
    FeaturePipeline = "featurepipelines"
    FeaturePipelineRun = "featurepipelineruns"
    Featureset = "featuresets"
    FeatureHistogram = "featurehistograms"
    LabelingPipeline = "labelingpipelines"
    LabelingPipelineRun = "labelingpipelineruns"
    Recipe = "recipes"
    RecipeRun = "reciperuns"
    SqlQuery = "sqlqueries"
    SqlQueryRun = "sqlqueryruns"
    WebRequest = "webrequests"
    WebRequestRun = "webrequestruns"

    CronPrediction = "cronpredictions"
    Prediction = "predictions"
    Predictor = "predictors"
    Curtain = "curtains"
    DataApp = "dataapps"

    Account = "accounts"
    Alert = "alerts"
    ApiToken = "apitokens"
    Connection = "connections"
    Commit = "commits"
    Lab = "labs"
    License = "licenses"
    Notifier = "notifiers"
    ServingSite = "servingsites"
    Tenant = "tenants"
    VirtualBucket = "virtualbuckets"
    VirtualCluster = "virtualclusters"
    VirtualVolume = "virtualvolumes"
    Attachment = "attachments"

    Model = "models"
    Notebook = "notebooks"
    NotebookRun = "notebookruns"
    ModelPipeline = "modelpipelines"
    ModelPipelineRun = "modelpipelineruns"
    ModelCompilerRun = "modelcompilerruns"
    ModelAutobuilder = "modelautobuilders"
    Report = "reports"
    CronReport = "cronreports"
    Study = "studies"

    Review = "reviews"
    PostMortem = "postmortems"
    RunBook = "runbooks"
    Todo = "todos"
    Meeting = "meetings"

    Secret = "secrets"
    Deployment = "deployments"
    Service = "services"
    Pod = "pods"
    Job = "jobs"



import grpc
from github.com.metaprov.modelaapi.pkg.apis.training.v1alpha1.generated_pb2 import ModelPipeline as MDModelPipeline
from github.com.metaprov.modelaapi.services.modelpipeline.v1.modelpipeline_pb2_grpc import ModelPipelineServiceStub
from github.com.metaprov.modelaapi.services.modelpipeline.v1.modelpipeline_pb2 import CreateModelPipelineRequest, \
    UpdateModelPipelineRequest, \
    DeleteModelPipelineRequest, GetModelPipelineRequest, ListModelPipelinesRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class ModelPipeline(Resource):
    def __init__(self, item: MDModelPipeline = MDModelPipeline(), client=None, namespace="", name="", version=Resource.DefaultVersion):
        super().__init__(item, client, namespace=namespace, name=name, version=version)


class ModelPipelineClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: ModelPipelineServiceStub = stub

    def create(self, modelpipeline: ModelPipeline) -> bool:
        request = CreateModelPipelineRequest()
        request.modelpipeline.CopyFrom(modelpipeline.raw_message)
        try:
            response = self.__stub.CreateModelPipeline(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, modelpipeline: ModelPipeline) -> bool:
        request = UpdateModelPipelineRequest()
        request.modelpipeline.CopyFrom(modelpipeline.raw_message)
        try:
            self.__stub.UpdateModelPipeline(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[ModelPipeline, bool]:
        request = GetModelPipelineRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetModelPipeline(request)
            return ModelPipeline(response.modelpipeline, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteModelPipelineRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteModelPipeline(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[ModelPipeline], bool]:
        request = ListModelPipelinesRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListModelPipelines(request)
            return [ModelPipeline(item, self) for item in response.modelpipelines.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





import grpc
from github.com.metaprov.modelaapi.pkg.apis.data.v1alpha1.generated_pb2 import Entity as MDEntity
from github.com.metaprov.modelaapi.services.entity.v1.entity_pb2_grpc import EntityServiceStub
from github.com.metaprov.modelaapi.services.entity.v1.entity_pb2 import CreateEntityRequest, \
    UpdateEntityRequest, \
    DeleteEntityRequest, GetEntityRequest, ListEntitiesRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class Entity(Resource):
    def __init__(self, item: MDEntity = MDEntity(), client=None, namespace="", name="", version=Resource.DefaultVersion):
        super().__init__(item, client, namespace=namespace, name=name, version=version)


class EntityClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: EntityServiceStub = stub

    def create(self, entity: Entity) -> bool:
        request = CreateEntityRequest()
        request.entity.CopyFrom(entity.raw_message)
        try:
            response = self.__stub.CreateEntity(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, entity: Entity) -> bool:
        request = UpdateEntityRequest()
        request.entity.CopyFrom(entity.raw_message)
        try:
            self.__stub.UpdateEntity(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[Entity, bool]:
        request = GetEntityRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetEntity(request)
            return Entity(response.entity, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteEntityRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteEntity(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[Entity], bool]:
        request = ListEntitiesRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListEntitys(request)
            return [Entity(item, self) for item in response.entitys.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





import grpc
from github.com.metaprov.modelaapi.pkg.apis.infra.v1alpha1.generated_pb2 import Account as MDAccount
from github.com.metaprov.modelaapi.services.account.v1.account_pb2_grpc import AccountServiceStub
from github.com.metaprov.modelaapi.services.account.v1.account_pb2 import CreateAccountRequest, \
    UpdateAccountRequest, \
    DeleteAccountRequest, GetAccountRequest, ListAccountsRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class Account(Resource):
    def __init__(self, item: MDAccount = MDAccount(), client=None, namespace="", name=""):
        super().__init__(item, client, namespace=namespace, name=name)


class AccountClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: AccountServiceStub = stub

    def create(self, account: Account) -> bool:
        request = CreateAccountRequest()
        request.account.CopyFrom(account.raw_message)
        try:
            response = self.__stub.CreateAccount(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, account: Account) -> bool:
        request = UpdateAccountRequest()
        request.account.CopyFrom(account.raw_message)
        try:
            self.__stub.UpdateAccount(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[Account, bool]:
        request = GetAccountRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetAccount(request)
            return Account(response.account, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteAccountRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteAccount(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[Account], bool]:
        request = ListAccountsRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListAccounts(request)
            return [Account(item, self) for item in response.accounts.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





import grpc
from github.com.metaprov.modelaapi.pkg.apis.infra.v1alpha1.generated_pb2 import ApiToken as MDApiToken
from github.com.metaprov.modelaapi.services.apitoken.v1.apitoken_pb2_grpc import ApiTokenServiceStub
from github.com.metaprov.modelaapi.services.apitoken.v1.apitoken_pb2 import CreateApiTokenRequest, \
    UpdateApiTokenRequest, \
    DeleteApiTokenRequest, GetApiTokenRequest, ListApiTokensRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class ApiToken(Resource):
    def __init__(self, item: MDApiToken = MDApiToken(), client=None, namespace="", name=""):
        super().__init__(item, client, namespace=namespace, name=name)


class ApiTokenClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: ApiTokenServiceStub = stub

    def create(self, apitoken: ApiToken) -> bool:
        request = CreateApiTokenRequest()
        request.apitoken.CopyFrom(apitoken.raw_message)
        try:
            response = self.__stub.CreateApiToken(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, apitoken: ApiToken) -> bool:
        request = UpdateApiTokenRequest()
        request.apitoken.CopyFrom(apitoken.raw_message)
        try:
            self.__stub.UpdateApiToken(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[ApiToken, bool]:
        request = GetApiTokenRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetApiToken(request)
            return ApiToken(response.apitoken, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteApiTokenRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteApiToken(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[ApiToken], bool]:
        request = ListApiTokensRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListApiTokens(request)
            return [ApiToken(item, self) for item in response.apitokens.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





import grpc
from github.com.metaprov.modelaapi.pkg.apis.team.v1alpha1.generated_pb2 import Todo as MDTodo
from github.com.metaprov.modelaapi.services.todo.v1.todo_pb2_grpc import TodoServiceStub
from github.com.metaprov.modelaapi.services.todo.v1.todo_pb2 import CreateTodoRequest, \
    UpdateTodoRequest, \
    DeleteTodoRequest, GetTodoRequest, ListTodosRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class Todo(Resource):
    def __init__(self, item: MDTodo = MDTodo(), client=None, namespace="", name=""):
        super().__init__(item, client, namespace=namespace, name=name)


class TodoClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: TodoServiceStub = stub

    def create(self, todo: Todo) -> bool:
        request = CreateTodoRequest()
        request.todo.CopyFrom(todo.raw_message)
        try:
            response = self.__stub.CreateTodo(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, todo: Todo) -> bool:
        request = UpdateTodoRequest()
        request.todo.CopyFrom(todo.raw_message)
        try:
            self.__stub.UpdateTodo(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[Todo, bool]:
        request = GetTodoRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetTodo(request)
            return Todo(response.todo, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteTodoRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteTodo(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[Todo], bool]:
        request = ListTodosRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListTodos(request)
            return [Todo(item, self) for item in response.todos.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





import grpc
from github.com.metaprov.modelaapi.pkg.apis.inference.v1alpha1.generated_pb2 import Prediction as MDPrediction
from github.com.metaprov.modelaapi.services.prediction.v1.prediction_pb2_grpc import PredictionServiceStub
from github.com.metaprov.modelaapi.services.prediction.v1.prediction_pb2 import CreatePredictionRequest, \
    UpdatePredictionRequest, \
    DeletePredictionRequest, GetPredictionRequest, ListPredictionsRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class Prediction(Resource):
    def __init__(self, item: MDPrediction = MDPrediction(), client=None, namespace="", name="", version=Resource.DefaultVersion):
        super().__init__(item, client, namespace=namespace, name=name, version=version)


class PredictionClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: PredictionServiceStub = stub

    def create(self, prediction: Prediction) -> bool:
        request = CreatePredictionRequest()
        request.prediction.CopyFrom(prediction.raw_message)
        try:
            response = self.__stub.CreatePrediction(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, prediction: Prediction) -> bool:
        request = UpdatePredictionRequest()
        request.prediction.CopyFrom(prediction.raw_message)
        try:
            self.__stub.UpdatePrediction(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[Prediction, bool]:
        request = GetPredictionRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetPrediction(request)
            return Prediction(response.prediction, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeletePredictionRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeletePrediction(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[Prediction], bool]:
        request = ListPredictionsRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListPredictions(request)
            return [Prediction(item, self) for item in response.predictions.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





import grpc
from github.com.metaprov.modelaapi.pkg.apis.training.v1alpha1.generated_pb2 import Notebook as MDNotebook
from github.com.metaprov.modelaapi.services.notebook.v1.notebook_pb2_grpc import NotebookServiceStub
from github.com.metaprov.modelaapi.services.notebook.v1.notebook_pb2 import CreateNotebookRequest, \
    UpdateNotebookRequest, \
    DeleteNotebookRequest, GetNotebookRequest, ListNotebooksRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class Notebook(Resource):
    def __init__(self, item: MDNotebook = MDNotebook(), client=None, namespace="", name="", version=Resource.DefaultVersion):
        super().__init__(item, client, namespace=namespace, name=name, version=version)


class NotebookClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: NotebookServiceStub = stub

    def create(self, notebook: Notebook) -> bool:
        request = CreateNotebookRequest()
        request.notebook.CopyFrom(notebook.raw_message)
        try:
            response = self.__stub.CreateNotebook(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, notebook: Notebook) -> bool:
        request = UpdateNotebookRequest()
        request.notebook.CopyFrom(notebook.raw_message)
        try:
            self.__stub.UpdateNotebook(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[Notebook, bool]:
        request = GetNotebookRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetNotebook(request)
            return Notebook(response.notebook, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteNotebookRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteNotebook(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[Notebook], bool]:
        request = ListNotebooksRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListNotebooks(request)
            return [Notebook(item, self) for item in response.notebooks.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





import grpc
from github.com.metaprov.modelaapi.pkg.apis.data.v1alpha1.generated_pb2 import FeaturePipeline as MDFeaturePipeline
from github.com.metaprov.modelaapi.services.featurepipeline.v1.featurepipeline_pb2_grpc import FeaturePipelineServiceStub
from github.com.metaprov.modelaapi.services.featurepipeline.v1.featurepipeline_pb2 import CreateFeaturePipelineRequest, \
    UpdateFeaturePipelineRequest, \
    DeleteFeaturePipelineRequest, GetFeaturePipelineRequest, ListFeaturePipelineRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class FeaturePipeline(Resource):
    def __init__(self, item: MDFeaturePipeline = MDFeaturePipeline(), client=None, namespace="", name="", version=Resource.DefaultVersion):
        super().__init__(item, client, namespace=namespace, name=name, version=version)


class FeaturePipelineClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: FeaturePipelineServiceStub = stub

    def create(self, featurepipeline: FeaturePipeline) -> bool:
        request = CreateFeaturePipelineRequest()
        request.featurepipeline.CopyFrom(featurepipeline.raw_message)
        try:
            response = self.__stub.CreateFeaturePipeline(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, featurepipeline: FeaturePipeline) -> bool:
        request = UpdateFeaturePipelineRequest()
        request.featurepipeline.CopyFrom(featurepipeline.raw_message)
        try:
            self.__stub.UpdateFeaturePipeline(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[FeaturePipeline, bool]:
        request = GetFeaturePipelineRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetFeaturePipeline(request)
            return FeaturePipeline(response.featurepipeline, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteFeaturePipelineRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteFeaturePipeline(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[FeaturePipeline], bool]:
        request = ListFeaturePipelineRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListFeaturePipelines(request)
            return [FeaturePipeline(item, self) for item in response.featurepipelines.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





import grpc
from github.com.metaprov.modelaapi.pkg.apis.data.v1alpha1.generated_pb2 import RecipeRun as MDRecipeRun
from github.com.metaprov.modelaapi.services.reciperun.v1.reciperun_pb2_grpc import RecipeRunServiceStub
from github.com.metaprov.modelaapi.services.reciperun.v1.reciperun_pb2 import CreateRecipeRunRequest, \
    UpdateRecipeRunRequest, \
    DeleteRecipeRunRequest, GetRecipeRunRequest, ListRecipeRunsRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class RecipeRun(Resource):
    def __init__(self, item: MDRecipeRun = MDRecipeRun(), client=None, namespace="", name="", version=Resource.DefaultVersion):
        super().__init__(item, client, namespace=namespace, name=name, version=version)


class RecipeRunClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: RecipeRunServiceStub = stub

    def create(self, reciperun: RecipeRun) -> bool:
        request = CreateRecipeRunRequest()
        request.reciperun.CopyFrom(reciperun.raw_message)
        try:
            response = self.__stub.CreateRecipeRun(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, reciperun: RecipeRun) -> bool:
        request = UpdateRecipeRunRequest()
        request.reciperun.CopyFrom(reciperun.raw_message)
        try:
            self.__stub.UpdateRecipeRun(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[RecipeRun, bool]:
        request = GetRecipeRunRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetRecipeRun(request)
            return RecipeRun(response.reciperun, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteRecipeRunRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteRecipeRun(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[RecipeRun], bool]:
        request = ListRecipeRunsRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListRecipeRuns(request)
            return [RecipeRun(item, self) for item in response.reciperuns.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





import grpc
from github.com.metaprov.modelaapi.pkg.apis.data.v1alpha1.generated_pb2 import DataProduct as MDDataProduct
from github.com.metaprov.modelaapi.services.dataproduct.v1.dataproduct_pb2_grpc import DataProductServiceStub
from github.com.metaprov.modelaapi.services.dataproduct.v1.dataproduct_pb2 import CreateDataProductRequest, \
    UpdateDataProductRequest, \
    DeleteDataProductRequest, GetDataProductRequest, ListDataProductsRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union

from modela.infra.VirtualBucket import VirtualBucket
from modela.infra.ServingSite import ServingSite
from modela.infra.Lab import Lab
import modela.data.models
from modela.data.models import DataProductSpec, Permissions, DataLocation
from modela.infra.models import NotificationSettings, Workload
from modela.training.common import TaskType


class DataProduct(Resource):
    def __init__(self, item: MDDataProduct = MDDataProduct(), client=None, namespace="", name="",
                 serving_site: Union[ServingSite, str] = None,
                 lab: Union[Lab, str] = None,
                 public: bool = None,
                 task_type: TaskType = None,
                 default_training_workload: Workload = None,
                 default_serving_workload: Workload = None,
                 default_bucket: Union[VirtualBucket, str] = None,
                 notification_settings: NotificationSettings = None,
                 permissions: Permissions = None):
        """
        :param client: The Data Product client repository, which can be obtained through an instance of Modela.
        :param namespace: The target namespace of the resource.
        :param name: The name of the resource.
        :param serving_site: The default Serving Site of the Data Product.
        :param lab: The default Lab of the Data Product.
        :param public: If enabled, the Data Product will be publicly accessible by all users without permissions.
        :param task_type: The default task type for child resources of the Data Product.
        :param default_training_workload: The default workload for training Jobs under the Data Product
        :param default_serving_workload: The default workload for model serving Jobs under the Data Product.
        :param default_bucket: The default bucket used for child resources of the Data Product.
        :param notification_settings: The default notification settings used for child resources of the Data Product.
        :param permissions: The permission specification that dictates which users can access the resources under
          the Data Product and what actions they can perform.
        """
        super().__init__(item, client, namespace=namespace, name=name)

        spec = self.spec
        if type(serving_site) == ServingSite:
            serving_site = serving_site.name
        spec.ServingSiteName = serving_site

        if type(lab) == Lab:
            lab = lab.name
        spec.LabName = lab

        if spec.Public is not None:
            spec.Public = public

        if task_type is not None:
            spec.Task = task_type

        if default_training_workload is not None:
            spec.TrainingResources = default_training_workload

        if default_serving_workload is not None:
            spec.ServingResources = default_serving_workload

        if default_bucket is not None:
            spec.DataLocation = DataLocation(BucketName=default_bucket)

        if notification_settings is not None:
            spec.Notification = notification_settings

        if permissions is not None:
            spec.Permissions = permissions

    @property
    def spec(self) -> DataProductSpec:
        return DataProductSpec().copy_from(self._object.spec).set_parent(self._object.spec)

    def default(self):
        DataProductSpec().apply_config(self._object.spec)


class DataProductClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: DataProductServiceStub = stub

    def create(self, dataproduct: DataProduct) -> bool:
        request = CreateDataProductRequest()
        request.dataproduct.CopyFrom(dataproduct.raw_message)
        try:
            response = self.__stub.CreateDataProduct(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, dataproduct: DataProduct) -> bool:
        request = UpdateDataProductRequest()
        request.dataproduct.CopyFrom(dataproduct.raw_message)
        try:
            self.__stub.UpdateDataProduct(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[DataProduct, bool]:
        request = GetDataProductRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetDataProduct(request)
            return DataProduct(response.dataproduct, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteDataProductRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteDataProduct(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[DataProduct], bool]:
        request = ListDataProductsRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListDataProducts(request)
            return [DataProduct(item, self) for item in response.dataproducts.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False





import os
import time

import grpc
from github.com.metaprov.modelaapi.pkg.apis.training.v1alpha1.generated_pb2 import Report as MDReport
from github.com.metaprov.modelaapi.services.report.v1.report_pb2_grpc import ReportServiceStub
from github.com.metaprov.modelaapi.services.report.v1.report_pb2 import CreateReportRequest, \
    UpdateReportRequest, DeleteReportRequest, GetReportRequest, ListReportsRequest, DownloadReportRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union

from modela.training.models import ReportSpec, ReportStatus
import tempfile
import webbrowser

class Report(Resource):
    def __init__(self, item: MDReport = MDReport(), client=None, namespace="", name=""):
        """
        :param client: The Report client repository, which can be obtained through an instance of Modela.
        :param namespace: The target namespace of the resource.
        :param name: The name of the resource.
        """

        super().__init__(item, client, namespace=namespace, name=name)

    @property
    def spec(self) -> ReportSpec:
        return ReportSpec().copy_from(self._object.spec)

    @property
    def status(self) -> ReportStatus:
        return ReportStatus().copy_from(self._object.status)

    def default(self):
        raise TypeError("Report {0} was not found; report resources cannot be created.".format(self.name))

    def submit(self):
        raise TypeError("Report resources cannot be created.")

    def update(self):
        raise TypeError("Report resources cannot be updated.")

    def download(self) -> bytes:
        self.ensure_client_repository()
        return self._client.download(self.namespace, self.name)

    def open_in_browser(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_file_path = os.path.join(temp_dir, '{0}.pdf'.format(self.name))
            with open(temp_file_path, 'wb+') as fh:
                fh.write(self.download())

            webbrowser.open('file://' + temp_file_path)
            time.sleep(1)



class ReportClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: ReportServiceStub = stub

    def create(self, report: Report) -> bool:
        raise TypeError("Modela currently does not support the creation of custom models.")
        # request = CreateReportRequest()
        # request.report.CopyFrom(report.raw_message)
        # try:
        #     response = self.__stub.CreateReport(request)
        #     return True
        # except grpc.RpcError as err:
        #     error = err
        #
        # ModelaException.process_error(error)
        # return False

    def update(self, report: Report) -> bool:
        raise TypeError("Report resources cannot be changed after creation.")
        # request = UpdateReportRequest()
        # request.report.CopyFrom(report.raw_message)
        # try:
        #     self.__stub.UpdateReport(request)
        #     return True
        # except grpc.RpcError as err:
        #     error = err
        #
        # ModelaException.process_error(error)
        # return False

    def get(self, namespace: str, name: str) -> Union[Report, bool]:
        request = GetReportRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetReport(request)
            return Report(response.report, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteReportRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteReport(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str, labels: dict = None) -> Union[List[Report], bool]:
        request = ListReportsRequest()
        request.namespace = namespace
        if labels is not None:
            request.labels.update(labels)

        try:
            response = self.__stub.ListReports(request)
            return [Report(item, self) for item in response.reports.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def download(self, namespace: str, name: str) -> bytes:
        request = DownloadReportRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.Download(request)
            return response.raw
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)


import grpc
from github.com.metaprov.modelaapi.pkg.apis.infra.v1alpha1.generated_pb2 import Connection as MDConnection
from github.com.metaprov.modelaapi.services.connection.v1.connection_pb2_grpc import ConnectionServiceStub
from github.com.metaprov.modelaapi.services.connection.v1.connection_pb2 import CreateConnectionRequest, \
    UpdateConnectionRequest, \
    DeleteConnectionRequest, GetConnectionRequest, ListConnectionsRequest

from modela.Resource import Resource
from modela.ModelaException import ModelaException
from typing import List, Union


class Connection(Resource):
    def __init__(self, item: MDConnection = MDConnection(), client=None, namespace="", name=""):
        super().__init__(item, client, namespace=namespace, name=name)


class ConnectionClient:
    def __init__(self, stub, modela):
        self.modela = modela
        self.__stub: ConnectionServiceStub = stub

    def create(self, connection: Connection) -> bool:
        request = CreateConnectionRequest()
        request.connection.CopyFrom(connection.raw_message)
        try:
            response = self.__stub.CreateConnection(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def update(self, connection: Connection) -> bool:
        request = UpdateConnectionRequest()
        request.connection.CopyFrom(connection.raw_message)
        try:
            self.__stub.UpdateConnection(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def get(self, namespace: str, name: str) -> Union[Connection, bool]:
        request = GetConnectionRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.GetConnection(request)
            return Connection(response.connection, self)
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def delete(self, namespace: str, name: str) -> bool:
        request = DeleteConnectionRequest()
        request.namespace = namespace
        request.name = name
        try:
            response = self.__stub.DeleteConnection(request)
            return True
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False

    def list(self, namespace: str) -> Union[List[Connection], bool]:
        request = ListConnectionsRequest()
        request.namespace = namespace
        try:
            response = self.__stub.ListConnections(request)
            return [Connection(item, self) for item in response.connections.items]
        except grpc.RpcError as err:
            error = err

        ModelaException.process_error(error)
        return False


